#include "monitor.h"
#include <QTimer>
#include <string>
#include <iostream>
#include <QBrush>
#include <sstream>
#include <QPicture>
#include "ui_monitor.h"
#include <QImage>
#include <QMouseEvent>
#include <QMessageBox>
#include <QtDebug>
using namespace rcss::net;
using namespace std;
Monitor::Monitor(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::Monitor)
{
    ui->setupUi(this);
    QTimer *timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this , SLOT(update()) );

//    connect(this,SIGNAL(repaint(),this,resize());
//    resize()

    timer->start(20);
    mode [0] = "beforeKickOff";
    mode [1] = "KickOff_Left";
    mode [2] = "KickOff_Right";
    mode [3] = "PlayOn";
    mode [4] = "KickIn_Left";
    mode [5] = "KickIn_Right";
    mode [6] = "corner_kick_left";
    mode [7] = "corner_kick_right";
    mode [8] = "goal_kick_left";
    mode [9] = "goal_kick_right";
    mode [10] ="offside_left offside_right";
    mode [11] ="GameOver";
    mode [12] ="Goal_Left";
    mode [13] ="Goal_Right";
    mode [14] ="free_kick_left";
    mode [15] ="free_kick_right";
    m_con = false;
    toggle = TEAM_Left ;

    playerNo = 9;
    fieldLength = 21.0;
    fieldWidth = 14.0;
    goalWidth = 2.1;
    goalDepth = 0.3;
    penaltyWidth = 3.9;
    penaltyLength = 1.8;
    monitorWidth = 400;
    monitorLength = 600;
    borderSize = 160;
    borderField=1;
    centerRadius=1;
    center = QPointF((borderSize+monitorLength)/2.0,(borderSize+monitorWidth)/2.0);
    scaleX = monitorLength / fieldLength;
    scaleY = -monitorWidth / fieldWidth;



}

QPointF Monitor::scale(QPointF realPos){
    QPointF ret(center.x()+realPos.x()*scaleX,center.y()+realPos.y()*scaleY);

    return ret;
}

QPointF Monitor::invertScale(QPointF scaledPos){
    QPointF ret((-center.x()+scaledPos.x())/scaleX,(-center.y()+scaledPos.y())/scaleY);

    return ret;
}

void Monitor::resize()
{
    qDebug() << "salam";
}

void Monitor::updateAll()
{

    if ( sock.connected() ){
        ui->Connect->setIcon(QIcon(":/pics/icons/disconnect.png"));
        ui->Connect->setText("Disconnect");
    }
    else{
        ui->Connect->setIcon(QIcon(":/pics/icons/connect.png"));
        ui->Connect->setText("Connect");
    }
}

Monitor::~Monitor()
{
    delete ui;
}

void Monitor::keyPressEvent(QKeyEvent *event)
{

    if ( event->text().toLower() == "b" )
    {
        dropBall();
    }
    if ( event->text().toLower() == "p" )
    {
        playOn();
    }
    if ( event->text().toLower() == "l" )
    {
        kickOffLeft();
    }
    if ( event->text().toLower() == "r" )
    {
        kickOffRight();
    }
    if ( event->text().toLower() == "m" )
    {
        toggle = !toggle;
    }
    if ( event->text().toLower() == "q" )
    {
        this->close();
    }

    for ( int i = 0 ; i < playerNo ; i++ )
    {
        if ( event->text().toInt() == i+1 )
        {
            dropPlayer(i+1);
        }
    }
}

void Monitor::paintEvent(QPaintEvent */*event*/)
{
    try
    {
        QPainter paint ;
        paint.begin(this);
        updateAll();
        string str;
        vector <nao> temp;
        static int t = 1 ;
        t *= -1 ;
        if ( sock.connected() )
        {
            str = sock.Receive();
            temp = parse.parse(str);
            parseThing(str);
            stringstream ss ;
            ss << t1 << " : " << p1 << " - " << p2 << " : " << t2 ;
            if ( ui->score->text().toStdString() != ss.str() )
                ui->score->setText( QString ( ss.str().c_str() ) );
            ui->mode->setText( QString ( cmode.c_str() ) );
            if ( t == 1 )
                ui->cycle->setProperty( "value" , cycle ) ;
        }
        drawField(paint);
        drawGoal(paint);
//        drawField(paint);
        drawOtherLines(paint);
        drawObjects (  paint ,  temp ) ;
        paint.end();
    }
    catch ( exception &e)
    {
        cout << e.what() << endl;
    }
}

void Monitor::on_Connect_clicked()
{

    try
    {
        if ( !sock.connected() )
        {
            string strHost = ui->Host->text().toStdString();
            int port = ui->Port->text().toInt();
            sock.Reconnect(Addr(port,strHost));
            usleep(100000);
            m_con=true;

        }
        else
        {
            sock.Disconnect();
            usleep(100000);
        }

    }
    catch ( exception &e )
    {
        cout << e.what() << endl;
    }
}

void Monitor::drawLeft (QPainter &paint , QPointF pos , double ang , int num )
{

    QPointF conv=scale(pos);
    paint.setPen(Qt::darkYellow);
    paint.setBrush(Qt::darkRed);
    paint.drawEllipse(conv , 8 , 8);
    paint.setPen(Qt::white);
    paint.drawLine(conv , conv + QPointF( 6*sin(gDegToRad(ang+180)) , 6*cos(gDegToRad(ang+180))) );
    paint.drawText( (conv).x()-4 , (conv).y()-2 , QString('0'+num) );
}

void Monitor::drawRight (QPainter &paint ,  QPointF pos , double ang , int num)
{

    QPointF conv=scale(pos);
    paint.setPen(Qt::darkRed);
    paint.setBrush(Qt::darkYellow);
    paint.drawEllipse(conv , 8 , 8);
    paint.setPen(Qt::white);
    paint.drawLine(conv , conv + QPointF( 6*sin(gDegToRad(ang+180)) , 6*cos(gDegToRad(ang+180))) );
    paint.drawText( (conv).x()-4 , (conv).y()-2 , QString('0'+num) );
}

////

void Monitor::drawLeftF (QPainter &paint , QPointF pos , double ang, int num )
{

    QPointF conv=scale(pos);
    paint.setPen(Qt::darkYellow);
    paint.setBrush(QColor(QRgb(0xF78888)));
    paint.drawEllipse(conv , 8 , 8);
    paint.setPen(Qt::white);
    paint.drawLine(conv , conv + QPointF( 6*sin(gDegToRad(ang+180)) , 6*cos(gDegToRad(ang+180))) );
    paint.drawText( (conv).x()-4 , (conv).y()-2 , QString('0'+num) );
}

void Monitor::drawRightF (QPainter &paint ,  QPointF pos , double ang , int num)
{
    QPointF conv=scale(pos);
    paint.setPen(Qt::darkRed);
    paint.setBrush(QColor(QRgb(0xDCCF5F)));
    paint.drawEllipse(conv , 8 , 8);
    paint.setPen(Qt::white);
    paint.drawLine(conv , conv + QPointF( 6*sin(gDegToRad(ang+180)) , 6*cos(gDegToRad(ang+180))) );
    paint.drawText( (conv).x()-4 , (conv).y()-2 , QString('0'+num) );
}

///

void Monitor::drawBall ( QPainter &paint ,  QPointF pos)
{
    QPointF conv=scale(pos);
    paint.setPen(Qt::white);
    paint.setBrush(Qt::blue);
    paint.drawEllipse(conv , 4.0 , 4.0);

}
void Monitor::dropBall()
{
    if ( sock.connected() )
    {
        QPointF drop_pos = invertScale(QCursor::pos()-QPointF(0,30));//-QPointF(borderSize,borderSize));// - geometry().topLeft() - center ;
//        drop_pos.set;
        stringstream ss ;
        ss << "(ball (pos " << drop_pos.x() << " " << drop_pos.y() << " 0.045)(vel 0 0 0))" ;
        sock.Send(ss.str());
    }
    else
    {
        cerr << "Heeey ! You Are Disconnected Can not send Message." << endl;
    }
}


void Monitor::drawGoal(QPainter &paint)
{

    QPointF LA2 =scale(QPointF(fieldLength/2+goalDepth,goalWidth/2));
    QPointF RA2 =scale(QPointF(fieldLength/2,-goalWidth/2));;
    QPointF LT1 =scale(QPointF(-fieldLength/2-goalDepth,goalWidth/2));;
    QPointF RT1 =scale(QPointF(-fieldLength/2,-goalWidth/2));;

//    qDebug() << LA2 << LT1 ;
    QRectF goalL(LA2 ,RA2 ) ;
    QRectF goalR(LT1 ,RT1 ) ;
    paint.setPen(Qt::white);
    paint.setBrush(Qt::yellow);
    paint.drawRect(goalL);
    paint.drawRect(goalR);
    paint.setBrush(Qt::transparent);
}

void Monitor::kickOffLeft()
{
    if ( sock.connected() )
    {
        stringstream ss ;
        ss << "(kickOff Left)" ;
        sock.Send(ss.str());
    }
    else
    {
        cerr << "Heeey ! You Are Disconnected Can not send Message." << endl;
    }
}

void Monitor::kickOffRight()
{
    if ( sock.connected() )
    {
        stringstream ss ;
        ss << "(kickOff Right)" ;
        sock.Send(ss.str());
    }
    else
    {
        cerr << "Heeey ! You Are Disconnected Can not send Message." << endl;
    }
}

void Monitor::playOn()
{
    if ( sock.connected() )
    {
        stringstream ss ;
        ss << "(playMode PlayOn)" ;
        sock.Send(ss.str());
    }
    else
    {
        cerr << "Heeey ! You Are Disconnected Can not send Message." << endl;
    }
}


void Monitor::parseThing( string msg )
{
    string str ;
    for (unsigned  int i = 0 ; i < msg.size() ; i++ )
    {
        if ( msg[i] != ')' && msg[i] != '(' )
            str.push_back(msg[i]);
        else
            str.push_back(' ');
    }

    int pos = str.find("team_left");
    string temp;
    string he ;
    stringstream ss ;
    if ( pos != string::npos )
    {
        temp = str.substr( pos , str.size()-1 ) ;
        ss.str(temp) ;
        ss >> he >> t1 ;
    }
    pos = str.find("team_right");
    if ( pos != string::npos )
    {
        temp = str.substr( pos , str.size()-1 ) ;
        ss.str (temp) ;
        ss >> he >> t2 ;
    }
    pos = str.find("score_left");
    if ( pos != string::npos )
    {
        temp = str.substr( pos , str.size()-1 ) ;
        ss.str (temp) ;
        ss >> he >> p1 ;
    }

    pos = str.find("score_right");
    if ( pos != string::npos )
    {
        temp = str.substr( pos , str.size()-1 ) ;
        ss.str (temp) ;
        ss >> he >> p2 ;
    }

    pos = str.find("time");
    if ( pos != string::npos )
    {
        temp = str.substr( pos , str.size()-1 ) ;
        ss.str (temp) ;
        ss >> he >> cycle ;
    }

    pos = str.find("play_modes");
    if ( pos != string::npos )
        pos = str.find("play_mode" , pos +1 );
    else
        pos = str.find("play_mode");
    if ( pos != string::npos )
    {
        temp = str.substr( pos , str.size()-1 ) ;
        ss.str (temp) ;
        int adad ;
        ss >> he >> adad ;
        cmode = mode [adad] ;
    }
}

void Monitor::drawOtherLines(QPainter &paint)
{
    /*


     sideUpLef         centerUp               sideUpRight
     ---------------------------------------------
     |                    |                      |
     |                    |                      |
     |                    |                      |
     |                    |                      |
     |                    |                      |
     |                    |                      |
     |                    |                      |
     ---------------------------------------------
     sideDownLef         centerDown               sideDownRight

     */
    QPointF centerUp  = scale(QPointF(0,-fieldWidth/2.0));
    QPointF centerDown  = scale(QPointF(0,fieldWidth/2.0));

    QPointF sideUpLeft  = scale(QPointF(-fieldLength/2.0,-fieldWidth/2.0));
    QPointF sideUpRight  = scale(QPointF(fieldLength/2.0,-fieldWidth/2.0));

    QPointF sideDownLeft  = scale(QPointF(-fieldLength/2.0,fieldWidth/2.0));
    QPointF sideDownRight  = scale(QPointF(fieldLength/2.0,fieldWidth/2.0));


    paint.setPen(Qt::white);

    /// Left  Side Of Fielde Drawing
    paint.drawLine(centerUp,centerDown);
    paint.drawLine( sideUpLeft ,sideUpRight );
    paint.drawLine( sideDownLeft , sideDownRight);
    paint.drawLine( sideUpLeft ,sideDownLeft );
    paint.drawLine( sideUpRight , sideDownRight);


    ///
    paint.drawEllipse(scale(QPointF(0,0)) , scaleX*centerRadius , scaleX*centerRadius );
    paint.setBrush(Qt::transparent);
    paint.drawEllipse(scale(QPointF(0,0)) , scaleX*(centerRadius+0.01) ,scaleX*(centerRadius+0.01) );

    paint.setBrush(Qt::white);
        paint.drawEllipse(scale(QPointF(0,0)) , 1 , 1 );

//    paint.drawEllipse(center , 0.05*z1 , 0.05*z1 );
//    paint.drawLine( center + QPointF ( 0 , 6*z2 ) , center + QPointF ( 0 , -6*z2 ) );
    paint.drawLine(centerUp,centerDown);

}


void Monitor::drawField(QPainter &paint)
{
    QPointF gushb =scale(QPointF(-fieldLength/2-borderField,-fieldWidth/2-borderField));
    QPointF gushp =scale(QPointF(fieldLength/2+borderField,fieldWidth/2+borderField));
    QRectF  field ( gushb , gushp ) ;
    paint.setPen(0x0021FF);
    paint.setBrush( QBrush(QColor(QRgb(0x3E974F))) );
    paint.drawRect ( field ) ;
}

void Monitor::drawObjects ( QPainter &paint, vector <nao> temp )
{
    if ( sock.connected() )
    {
        for (unsigned int i = 0 ; i < temp.size() ; i++)
        {
            QPointF pp(temp[i].pos.x(),temp[i].pos.y()) ;
            if ( temp[i].team == 1 ) {
                if ( temp[i].pos.z() > 0.20 )
                    drawLeft( paint , pp , temp[i].rot.z() , temp[i].num );
                else
                    drawLeftF( paint , pp , temp[i].rot.z() , temp[i].num );
            }
            else if ( temp[i].team == 2 ) {
                if ( temp[i].pos.z() > 0.20 )
                    drawRight(paint , pp , temp[i].rot.z() , temp[i].num );
                else
                    drawRightF(paint , pp , temp[i].rot.z() , temp[i].num );
            }
        }
        drawBall(paint ,QPointF(parse.getBallPos().x(),parse.getBallPos().y()));
    }
}

void Monitor::dropPlayer( int player )
{
    if ( sock.connected() )
    {
        QPointF drop_pos = invertScale(QCursor::pos()-QPointF(0,30));
        stringstream ss;
        if ( toggle == TEAM_Left )
            ss << "(agent (team Left)(unum " << player << ")(pos " << drop_pos.x() << " " << drop_pos.y() << " 0.45))" ;
        if ( toggle == TEAM_Right )
            ss << "(agent (team  Right)(unum " << player << ")(pos " << drop_pos.x() << " " << drop_pos.y() << " 0.45))" ;
        sock.Send(ss.str());
    }
    else
    {
        cerr << "Heeey ! You Are Disconnected Can not send Message." << endl;
    }
}

void Monitor::on_help_clicked()
{
    QMessageBox::about(this,qApp->tr("Help!") ,
                        qApp->tr(
                            "rcsnao2d current keys are :\n\n"
                            "   q : quit program\n"
                            "   p: change playmode to playon \n"
                            "   l: change playmode to KickOffLeft \n"
                            "   r: changle playmode to kickOffRight \n"
                            "   b: drop ball in mouse position \n"
                            "   m: toggle current selected team \n"
                            "   number 1...10 : move player in current selected team to mouse position\n"
                        )
                       );
}
